const AUR_SUPPORT = false;
/**********************************************************
 * Enable the flag above the get AUR support on Arch      *
 * Disable to use packageKit instead, without AUR support *
 **********************************************************/

const St = imports.gi.St;
const Main = imports.ui.main;
const Search = imports.ui.search;
const SearchDisplay = imports.ui.searchDisplay;
const IconGrid = imports.ui.iconGrid;
const GLib = imports.gi.GLib;
const Util = imports.misc.util;


// FIXME: sometimes only one row is shown
const MAX_SEARCH_RESULTS_ROWS = 3; // Maxnumber of rows shown
// little icon to save space for the description and version number
const ICON_SIZE = 32;

let appProvider = "";

function AppResult(result) {
    this._init(result);
}


AppResult.prototype = {
    _init: function(resultMeta) {
        // contact style icon, so full app name can be displayed
        this.actor = new St.Bin({ style_class: 'contact',
                                  reactive: true,
                                  track_hover: true });

        let content = new St.BoxLayout( { style_class: 'contact-content',
                                          vertical: false });
        this.actor.set_child(content);
        // create the icon
        let icon = new St.Icon({ icon_type: St.IconType.FULLCOLOR,
                                 icon_size: ICON_SIZE,
                                 icon_name: 'emblem-package',
                                 style_class: 'contact-icon' });
        // Add icon to content
        content.add(icon, { x_fill: true,
                            y_fill: false,
                            x_align: St.Align.START,
                            y_align: St.Align.START });
        // create text to be displayed
        let result = new St.BoxLayout({ style_class: 'contact-details',
                                        vertical: true });
        // add text to content
        content.add(result, { x_fill: true, x_align: St.Align.START });
        // create top label
        let exprLabel = new St.Label({ text: resultMeta.expr,
                                         style_class: 'result-expr' });
        // create second label
        let resultLabel = new St.Label({ text: resultMeta.result,
                                         style_class: 'result-result' });
        // add labels to text
        result.add(exprLabel, { x_fill: false, x_align: St.Align.START });
        result.add(resultLabel, { x_fill: false, x_align: St.Align.START });
    }

};

function AppProvider() {
    this._init.apply(this, arguments);
}

AppProvider.prototype = {
    __proto__: Search.SearchProvider.prototype,

    _init: function(title) {
        Search.SearchProvider.prototype._init.call(this, title);
    },
    
    _validExpression: function(expression) {
        // Only start to search for longer strings to minimize 
        // latency when it's not needed
        return expression.length>3;
    },

    getInitialResultSet: function(terms) {
        let expr = "";
        // FIXME: spaces and dashes aren't handled very well
        // get the different search terms
        for(let i=0; i<terms.length; i++) {
           expr += terms[i];
        }
        if (this._validExpression(expr)) {
            try {
                if (AUR_SUPPORT) {
                    // For AUR support, use package-query to search
                    // To minimize time, only search the repositories first
                    let [success, out, err, error] = GLib.spawn_sync(null,
                                 ["package-query", "-Ss", expr], null, 4, null);
                    var i = 0;
                    var resultSet = new Array();
                    // get all the results from the repositories in 
                    // the resultSet
                    if(error == 0 && out.length != 0) {
                        var outputSet =  out.toString().split("\n");
                        for (i = 0; i < (outputSet.length-1)/2; i++) {
                                resultSet[i] = {
                                        'expr': outputSet[2*i].split("/")[0], 
                                        'result':outputSet[2*i].split("/")[1]
                                        };
                        }
                    }
                    if ( i < 12 ) {
                        // If few results are shown, also search the AUR
                        let [success, out, err, error] = GLib.spawn_sync(null, 
                                ["package-query", "-As", expr], null, 4, null);
                        if(error == 0 && out.length != 0) {
                            outputSet =  out.toString().split("\n");
                            // Add results from the AUR to the resultSet
                            for (j = 0; j < (outputSet.length-1)/2; j++) {
                                resultSet[i+j] = {
                                        'expr': outputSet[2*j].split("/")[0], 
                                        'result':outputSet[2*j].split("/")[1]
                                        };
                            }
                        }
                    }
                       
                    return resultSet;
                } else {
                    // Without AUR support, use packageKit
                    let [success, out, err, error] = GLib.spawn_sync(null, 
                            ["pkcon", 
                                        "--filter=~installed", 
                                        "search", 
                                        "name", 
                                         expr], 
                            null, 4, null);
                    var resultSet = new Array();
                    if( error == 0 && out.length != 0) {
                        var outputSet =  out.toString().split("\n");
                        // get all the results from the repositories in 
                        // the resultSet
                        for (       i = 0; 
                                    i+2 < outputSet.length && 
                                    outputSet[i+2].split("\t").length > 1 ; 
                                    i++) 
                        {
                            resultSet[i] = {
                                        'expr': "INSTALL", 
                                        'result':outputSet[i+2].split("\t")[1]};
                        }
                    }
                    return resultSet;
                }
				
            }
            catch(exp) {
                // If there is an error, return a result with the error 
                // as message. The complete error can be viewed with the
                // looking glass
                return [{'expr':"EXCEPTION", 'result': exp.toString()}];
            }
        }
        else {
            return [];
        }
    },

    getSubsearchResultSet: function(prevResults, terms) {
        return this.getInitialResultSet(terms);
    },

    getResultMeta: function(result) {
        return {
            'id': result.result,
            'result': result.result,
            'expr': result.expr
        };
    },

    createResultActor: function(resultMeta, terms) {
        let result = new AppResult(resultMeta);
        return result.actor;
    },

    createResultContainerActor: function() {
        let grid = new IconGrid.IconGrid({ rowLimit: MAX_SEARCH_RESULTS_ROWS,
                                           xAlign: St.Align.START });
        grid.actor.style_class = 'contact-grid';

        let actor = new SearchDisplay.GridSearchResults(this, grid);
        return actor;
    },

    activateResult: function(resultId) {
        if (AUR_SUPPORT) {
            // If AUR support is needed, use packer to install the package
            Util.spawnCommandLine("gnome-terminal -x packer -S "+ 
                                    resultId.split(" ")[0] +" --noedit");
        } else {
            // If there is no AUR support, parse the packagename from the 
            // result, and install the package using Gnome-PackageKit
            var packageNameArray = resultId.split("-");
            var pattern = new RegExp("[0-9]");
            var i = 1;
            var packageName =packageNameArray[0];
            while (i<packageNameArray.length && 
                                !pattern.test(packageNameArray[i].charAt(0))){
                packageName += "-" + packageNameArray[i];
                i++;
            }
            Util.spawnCommandLine("gpk-install-package-name "+ packageName );
        }
    }
}

function init() {
    appProvider = new AppProvider('App Search');
}

function enable() {
    Main.overview.addSearchProvider(appProvider);
}

function disable() {
    Main.overview.removeSearchProvider(appProvider);
}
